Type: Package
Package: rvg
Title: R Graphics Devices for 'Office' Vector Graphics Output
Version: 0.4.2
Authors@R: c(
    person("David", "Gohel", , "david.gohel@ardata.fr", role = c("aut", "cre")),
    person("ArData", role = "cph"),
    person("Bob", "Rudis", role = "ctb",
           comment = "the javascript code used by function set_attr"),
    person("Francois", "Brunetti", , "francois.brunetti@lysis-consultants.fr", role = "ctb",
           comment = "clipping algorithms")
  )
Description: Vector Graphics devices for 'Microsoft PowerPoint' and
    'Microsoft Excel'. Functions extending package 'officer' are provided
    to embed 'DrawingML' graphics into 'Microsoft PowerPoint'
    presentations and 'Microsoft Excel' workbooks.
License: GPL-3
URL: https://ardata-fr.github.io/officeverse/,
        https://davidgohel.github.io/rvg/
BugReports: https://github.com/davidgohel/rvg/issues
Depends: R (>= 3.0)
Imports: gdtools (>= 0.5.0), grDevices, officer (>= 0.7.4), Rcpp (>=
        0.12.12), rlang, systemfonts, xml2 (>= 1.0.0)
Suggests: grid, testthat
LinkingTo: Rcpp, systemfonts
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: libpng
NeedsCompilation: yes
Packaged: 2026-04-24 12:17:54 UTC; davidgohel
Author: David Gohel [aut, cre],
  ArData [cph],
  Bob Rudis [ctb] (the javascript code used by function set_attr),
  Francois Brunetti [ctb] (clipping algorithms)
Maintainer: David Gohel <david.gohel@ardata.fr>
Repository: CRAN
Date/Publication: 2026-04-24 13:30:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-24 16:32:14 UTC; unix
Archs: rvg.so.dSYM

Type: Package
Package: rvg
Title: R Graphics Devices for 'Office' Vector Graphics Output
Version: 0.4.2
Authors@R: c(
    person("David", "Gohel", , "david.gohel@ardata.fr", role = c("aut", "cre")),
    person("ArData", role = "cph"),
    person("Bob", "Rudis", role = "ctb",
           comment = "the javascript code used by function set_attr"),
    person("Francois", "Brunetti", , "francois.brunetti@lysis-consultants.fr", role = "ctb",
           comment = "clipping algorithms")
  )
Description: Vector Graphics devices for 'Microsoft PowerPoint' and
    'Microsoft Excel'. Functions extending package 'officer' are provided
    to embed 'DrawingML' graphics into 'Microsoft PowerPoint'
    presentations and 'Microsoft Excel' workbooks.
License: GPL-3
URL: https://ardata-fr.github.io/officeverse/,
        https://davidgohel.github.io/rvg/
BugReports: https://github.com/davidgohel/rvg/issues
Depends: R (>= 3.0)
Imports: gdtools (>= 0.5.0), grDevices, officer (>= 0.7.4), Rcpp (>=
        0.12.12), rlang, systemfonts, xml2 (>= 1.0.0)
Suggests: grid, testthat
LinkingTo: Rcpp, systemfonts
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: libpng
NeedsCompilation: yes
Packaged: 2026-04-24 12:17:54 UTC; davidgohel
Author: David Gohel [aut, cre],
  ArData [cph],
  Bob Rudis [ctb] (the javascript code used by function set_attr),
  Francois Brunetti [ctb] (clipping algorithms)
Maintainer: David Gohel <david.gohel@ardata.fr>
Repository: CRAN
Date/Publication: 2026-04-24 13:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 10:37:57 UTC; unix
Archs: rvg.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(ph_with,dml)
S3method(sheet_add_drawing,dml)
export(body_add_vg)
export(dml)
export(dml_pptx)
export(dml_xlsx)
export(xl_add_vg)
import(systemfonts)
importFrom(Rcpp,sourceCpp)
importFrom(gdtools,font_family_exists)
importFrom(gdtools,font_set_auto)
importFrom(grDevices,dev.off)
importFrom(officer,fortify_location)
importFrom(officer,ph_location)
importFrom(officer,ph_location_type)
importFrom(officer,ph_with)
importFrom(officer,sheet_add_drawing)
importFrom(officer,xlsx_drawing)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(xml2,as_xml_document)
importFrom(xml2,read_xml)
importFrom(xml2,xml_add_child)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
useDynLib(rvg,.registration = TRUE)

rvg package
================

<!-- badges: start -->

[![R-CMD-check](https://github.com/davidgohel/rvg/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/davidgohel/rvg/actions/workflows/R-CMD-check.yaml)
[![version](https://www.r-pkg.org/badges/version/rvg)](https://CRAN.R-project.org/package=rvg)
![cranlogs](https://cranlogs.r-pkg.org/badges/rvg) <!-- badges: end -->

`rvg` provides two graphics devices that produce Vector Graphics outputs
in DrawingML format for Microsoft PowerPoint with `dml_pptx()` and for
Microsoft Excel with `dml_xlsx()`. These formats let users edit the
graphic elements (*editable graphics*) within PowerPoint or Excel and
have a very good rendering.

These raw XML outputs cannot be used *as is*. The `dml()` function is a
simple wrapper that marks plot instructions as Vector Graphics
instructions, so they can be added to an Office document via `officer`:

- in PowerPoint with `officer::ph_with()`
- in Excel with `officer::sheet_add_drawing()`

``` r
library(rvg)
library(ggplot2)
library(officer)

my_vec_graph <- dml(code = barplot(1:5, col = 2:6))

doc <- read_pptx()
doc <- add_slide(doc, layout = "Title and Content", master = "Office Theme")
doc <- ph_with(doc, my_vec_graph, location = ph_location_fullsize())
print(doc, target = "my_plot.pptx")
```

For Excel workbooks, pass a `dml()` object to
`officer::sheet_add_drawing()`:

``` r
doc <- read_xlsx()
doc <- sheet_add_drawing(doc, sheet = "Feuil1",
  value = dml(code = barplot(1:5, col = 2:6)),
  left = 1, top = 2, width = 6, height = 6)
print(doc, target = "vg.xlsx")
```

By default, these graphics are *editable*; element edition can be
disabled with option `editable = FALSE`.

Note that `xl_add_vg()` will be deprecated in favor of
`officer::sheet_add_drawing()`.

## Installation

You can get the development version from GitHub:

``` r
devtools::install_github("davidgohel/rvg")
```

Install the CRAN version:

``` r
install.packages("rvg")
```

# &#127922; rvg &#127922; - Random Values Generator
![Testing](https://github.com/zehanort/rvg/workflows/Testing/badge.svg)
[![codecov](https://codecov.io/gh/zehanort/rvg/branch/dev/graph/badge.svg)](https://codecov.io/gh/zehanort/rvg)
[![CodeFactor](https://www.codefactor.io/repository/github/zehanort/rvg/badge)](https://www.codefactor.io/repository/github/zehanort/rvg)
![PyPI release](https://img.shields.io/pypi/v/rvg?label=PyPI%20release)

## Description

`rvg` is a Python 3 package utility to create random values of any Python 3 data type.

Its main purpose is to help in applications where reliable -in terms of type safety- random values are needed (e.g. statistics, machine learning, general testing etc), and in specific layouts (e.g. "I want a `numpy` structured array of random pairs of ints and floats and I want it now").

## Authors
[Sotiris Niarchos](https://github.com/zehanort) and [George Papadopoulos](https://github.com/gepapado)

## Installation

You can either:
- download the source code from the [releases page](https://github.com/zehanort/rvg/releases) or clone the repo (the `master` branch will always mirror the latest release)
- use `pip`:
```
pip install rvg
```

## Current Status

### Alpha release

For the time being, only `numpy` types are supported. More specifically:
- `numpy` scalar data types
- `numpy` arrays of scalar data types
- `numpy` arrays of structured data types

After the alpha release, more features will be implemented, focusing mainly on Python 3 native types.

## Usage

Right now, `rvg` provides 2 interafaces for random values generation through the `NumPyRVG` class:
1. Create a generator of a *certain data type*
2. Create a generator with *specific numerical limits*
A demonstration follows:

```Python
from rvg import NumPyRVG
import numpy as np

# Interface 1
randuint = NumPyRVG(dtype=np.uint16)

# Interface 2
randsmall = NumPyRVG(limit=10) # same as limits=(-10, 10)
randbig = NumPyRVG(limits=(1e10, 1e100))
```

The functionalities of `NumPyRVG` include the generation of:

- `numpy` scalar data types:

```Python console
>>> randsmall(np.uint8)
8
>>> randbig(np.double)
1.9296971162995923e+99
>>> res = [randsmall(t) for t in [np.int8, np.uint16, np.float32, np.double]]
>>> res
[7, 1, 3.0503626, 3.759943941132951]
>>> list(map(type, res))
[<class 'numpy.int8'>, <class 'numpy.uint16'>, <class 'numpy.float32'>, <class 'numpy.float64'>]
```

- `numpy` array data types from scalar types:

```Python console
>>> randuint((50, 100), shape=3)
array([79, 81, 85], dtype=uint16)
>>> randsmall(np.float16, shape=(4, 2))
array([[-7.23 , -9.31 ],
       [-4.97 , -6.06 ],
       [-5.19 , -3.344],
       [-6.586, -3.133]], dtype=float16)
```

- `numpy` structured array data types (structured datatypes can be nested). Limits and shapes can be given in the form of a dictionary, describing all limits and/or shapes of each field of each level of the structured data type. An example follows:

```Python
#  Consider the struct definition below, in C:
##############################################
#  typedef struct knode {
#      int location;
#      int indices [3];
#      int  keys [3];
#      bool is_leaf;
#      int num_keys;
#  } knode;
##############################################

import numpy as np
from rvg import NumPyRVG

knode = np.dtype([
    ('location', int),
    ('indices', (int, 3)),
    ('keys', (int, 3)),
    ('is_leaf', int),
    ('num_keys', int)
])

knode_params = {
    'location'  : (0, 10),
    'indices'   : 42,
    'keys'      : 117,
    'is_leaf'   : (0, 2),
    'num_keys'  : (0, 256)
}

random_knode = NumPyRVG(dtype=knode)
knodes_array = random_knode(knode_params, 5)
print(knodes_array)

```
The output of the above script is a nested structured `numpy` array consisting of 5 `knode` structs, randomly initialized!

```
[(0, [-18, -11,  34], [  89,   35,  -57], 1, 189)
 (6, [-35,   0,   4], [ -56,  -65,   26], 1, 217)
 (4, [-29,  40,  37], [  93, -116,   91], 0,  38)
 (2, [-28, -42, -36], [-101,    0,  -43], 0,  82)
 (0, [-14, -19, -13], [ -98,  -46,  -78], 1, 238)]
```
If the script was run in an interpreter, you could inspect its type as well:

```Python console
>>> knodes_array
array([(0, [-18, -11,  34], [  89,   35,  -57], 1, 189),
       (6, [-35,   0,   4], [ -56,  -65,   26], 1, 217),
       (4, [-29,  40,  37], [  93, -116,   91], 0,  38),
       (2, [-28, -42, -36], [-101,    0,  -43], 0,  82),
       (0, [-14, -19, -13], [ -98,  -46,  -78], 1, 238)],
      dtype=[('location', '<i8'), ('indices', '<i8', (3,)), ('keys', '<i8', (3,)), ('is_leaf', '<i8'), ('num_keys', '<i8')])
```
The feature of nesting is not limited in arrays; you can create data types that are as complex as you want and/or need! See the relative [test](https://github.com/zehanort/rvg/blob/master/tests/test_NumPyRVG_class_with_types.py) as an example of struct nesting.

library(tidyverse)
library(rvg)
library(officer)


plot <- ggplot(mtcars, aes(x = disp, y = mpg)) +
  geom_point()

plot_rvg <- dml(ggobj = plot)

ppt_template <- read_pptx() %>%
  add_slide() %>%
  ph_with(plot, location = ph_location(left = 1.09, top = 3.03, height = 3.7, width = 9.4)) %>%
  add_slide() %>%
  ph_with(plot_rvg, location = ph_location(left = 1.09, top = 3.03, height = 3.7, width = 9.4))

print(ppt_template, tempfile(fileext = ".pptx")) |> browseURL()


ppt_default <- read_pptx() %>%
  add_slide(layout = "Title and Content", master = "Office Theme") %>%
  ph_with(plot_rvg, location = ph_location_type())

print(ppt_default, tempfile(fileext = ".pptx")) |> browseURL()


# -- Test compound paths (donut with hole) ------------------------------------
# Uses polypath with EvenOdd rule: outer circle filled, inner circle cut out

angles <- seq(0, 2 * pi, length.out = 100)[-1]
donut <- data.frame(
  x = c(cos(angles) * 3, cos(rev(angles)) * 1.5),
  y = c(sin(angles) * 3, sin(rev(angles)) * 1.5),
  group = 1
)

plot_path <- ggplot(donut, aes(x, y, group = group)) +
  geom_polygon(fill = "steelblue", colour = "black") +
  # polypath-based donut via geom_polygon with a hole subgroup
  coord_equal() +
  theme_minimal() +
  ggtitle("Compound path: donut (hole should be visible)")

# Alternative using explicit path with geom from ggforce
# or a simpler polygon-with-hole example:
plot_path2 <- ggplot() +
  geom_polygon(
    data = data.frame(
      x = c(0, 10, 10, 0, 3, 3, 7, 7),
      y = c(0, 0, 10, 10, 3, 7, 7, 3),
      subid = c(rep(1, 4), rep(2, 4))
    ),
    aes(x = x, y = y, subgroup = subid),
    fill = "coral", colour = "black"
  ) +
  coord_equal() +
  theme_minimal() +
  ggtitle("Compound path: square with hole")

# PPTX output
pptx_path_test <- read_pptx() %>%
  add_slide(layout = "Title and Content", master = "Office Theme") %>%
  ph_with(dml(ggobj = plot_path), location = ph_location_type()) %>%
  add_slide(layout = "Title and Content", master = "Office Theme") %>%
  ph_with(dml(ggobj = plot_path2), location = ph_location_type())

print(pptx_path_test, tempfile(fileext = ".pptx")) |> browseURL()

# XLSX output
wb <- read_xlsx()
wb <- xl_add_vg(wb, sheet = "Feuil1",
  code = print(plot_path2),
  width = 6, height = 6, left = 1, top = 1)
print(wb, "test_compound_path.xlsx")

# -- Test fill opacity / noFill (#52) -----------------------------------------
# Points with alpha but no explicit color: fill should be semi-transparent
# Points with explicit fill: fill alpha should be preserved
# Open shapes (pch 1): should have no fill (transparent)

plot_alpha <- ggplot(mtcars[1:3, ], aes(wt, mpg)) +
  geom_point(alpha = 0.5, size = 10) +
  geom_point(aes(y = mpg - 2), alpha = 0.5, size = 10, colour = "red") +
  geom_point(aes(y = mpg - 4), alpha = 0.5, size = 10, shape = 21,
             fill = "steelblue", colour = "black") +
  geom_point(aes(y = mpg - 6), size = 10, shape = 1, colour = "black") +
  theme_minimal() +
  ggtitle("Alpha test: filled, colored, shape 21, open circle")

pptx_alpha <- read_pptx() |>
  add_slide(layout = "Title and Content", master = "Office Theme") |>
  ph_with(dml(ggobj = plot_alpha), location = ph_location_type())

print(pptx_alpha, tempfile(fileext = ".pptx")) |> browseURL()

# -- Test circle clipping (#61) -----------------------------------------------
# Point at (3,1) is outside xlim [0,2]: should be hidden entirely.
# Point at (1.8,1) has a large size: center is inside but circle edge
# extends past the right limit — it should still be visible.

plot_circle_clip <- ggplot(
  data.frame(x = c(1, 1.8, 3), y = 1, sz = c(5, 30, 5)),
  aes(x, y, size = sz)
) +
  geom_point(colour = "steelblue") +
  scale_size_identity() +
  coord_cartesian(xlim = c(0, 2), ylim = c(0, 2), expand = FALSE) +
  theme_minimal() +
  ggtitle("Circle clip: (3,1) hidden, (1.8,1) large visible, (1,1) normal")

pptx_clip_test <- read_pptx() |>
  add_slide(layout = "Title and Content", master = "Office Theme") |>
  ph_with(dml(ggobj = plot_circle_clip), location = ph_location_type())

print(pptx_clip_test, tempfile(fileext = ".pptx")) |> browseURL()

#' @export
#' @title add a plot output as vector graphics into an Excel object
#' @description
#' Deprecated, use [sheet_add_drawing()] with a [dml()] object instead.
#' @param x an \code{rxlsx} object produced by \code{officer::read_xlsx}
#' @param code plot instructions
#' @param sheet sheet label/name
#' @param height,width Height and width in inches.
#' @param left,top left and top origin of the plot on the sheet in inches.
#' @param ... arguments passed on to \code{\link{dml_xlsx}}.
#' @examples
#' \donttest{
#' library(officer)
#' my_ws <- read_xlsx()
#' my_ws <- sheet_add_drawing(my_ws,
#'   value = dml(code = barplot(1:5, col = 2:6)),
#'   sheet = "Feuil1",
#'   width = 6, height = 6, left = 1, top = 2
#' )
#' fileout <- tempfile(fileext = ".xlsx")
#' print(my_ws, target = fileout)
#' }
xl_add_vg <- function(x, sheet, code, left, top, width, height, ...) {
  .Deprecated("sheet_add_drawing")
  sheet_add_drawing(
    x,
    value = dml(code = code, ...),
    sheet = sheet,
    left = left,
    top = top,
    width = width,
    height = height
  )
}

#' @title DrawingML graphic device for Microsoft Excel
#'
#' @description
#' Graphics devices for Microsoft Excel DrawingML format.
#'
#' @param file the file where output will appear.
#' @param height,width Height and width in inches.
#' @param offx,offy top and left origin of the plot
#' @param bg Default background color for the plot (defaults to "white").
#' @param fonts Named list of font names to be aliased with
#'   fonts installed on your system. If unspecified, the R default
#'   families `sans`, `serif`, `mono` and `symbol` are aliased
#'   to the families detected by [gdtools::font_set_auto()].
#'
#'   **Important:** fonts are referenced by name in the DrawingML output
#'   and are **not embedded**. The reader of the Excel file must also
#'   have the same fonts installed, otherwise Office will substitute them.
#'   For maximum portability, prefer fonts bundled with Microsoft Office
#'   such as `"Arial"`, `"Times New Roman"` and `"Courier New"`.
#'
#'   Fonts must be installed on your system; use
#'   `gdtools::font_family_exists()` to verify availability.
#'
#'   Example: `list(sans = "Arial", serif = "Times New Roman", mono = "Courier New")`.
#' @param pointsize default point size.
#' @param editable should vector graphics elements (points, text, etc.) be editable.
#' @param id specifies a unique identifier (integer) within the
#' sheet that will contain the DrawingML instructions.
#' @param last_rel_id specifies the last unique identifier (integer)
#' within relationship file that will be used to reference embedded
#' raster images if any.
#' @param standalone produce a standalone drawingml file? If \code{FALSE}, omits
#'   xml header and namespaces.
#' @examples
#' \donttest{
#' dml_xlsx(file = tempfile())
#' plot(1:11, (-5:5)^2, type = "b", main = "Simple Example")
#' dev.off()
#' }
#' @keywords device
#' @seealso \code{\link{Devices}}
#' @export
dml_xlsx <- function(
  file = "Rplots.dml",
  width = 6,
  height = 6,
  offx = 1,
  offy = 1,
  bg = "white",
  fonts = list(),
  pointsize = 12,
  editable = TRUE,
  id = 1L,
  last_rel_id = 1L,
  standalone = TRUE
) {
  img_directory <- tempfile()
  dir.create(img_directory, recursive = TRUE, showWarnings = FALSE)
  raster_prefix <- paste0(img_directory, "/raster-")

  system_fonts <- validate_fonts(fonts)
  invisible(XLSX_(
    file,
    bg,
    width,
    height,
    offx = offx,
    offy = offy,
    pointsize = pointsize,
    aliases = list(system = system_fonts),
    editable = editable,
    id = id,
    raster_prefix = raster_prefix,
    last_rel_id = last_rel_id,
    standalone = standalone
  ))
}

context("XLSX raster generation")
library(grid)
library(xml2)

test_that("raster is generated", {
  file <- tempfile()
  dml_xlsx(file = file, bg = "transparent")
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp1"))
  grid.rect()
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp2", angle = 10))
  redGradient <- matrix(hcl(0, 80, seq(50, 80, 10)), nrow = 4, ncol = 5)
  grid.raster(redGradient)
  dev.off()

  xml_raw <- paste0(readLines(file, warn = FALSE), collapse = "")
  m <- regmatches(
    xml_raw,
    regexpr("<!-- rvg_raster_prefix:(.+?) -->", xml_raw, perl = TRUE)
  )
  expect_length(m, 1L)
  raster_prefix <- sub(
    "<!-- rvg_raster_prefix:(.+?) -->",
    "\\1",
    m,
    perl = TRUE
  )
  raster_files <- list.files(
    path = dirname(raster_prefix),
    pattern = paste0("^", basename(raster_prefix), "(.*)\\.png$"),
    full.names = TRUE
  )
  expect_equal(length(raster_files), expected = 1)
})


test_that("pic tag can be found", {
  file <- tempfile()

  dml_xlsx(file = file, bg = "transparent")
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp1"))
  grid.rect()
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp2", angle = 10))
  redGradient <- matrix(hcl(0, 80, seq(50, 80, 10)), nrow = 4, ncol = 5)
  grid.raster(redGradient)
  dev.off()

  doc <- read_xml(file)
  pic_node <- xml_find_first(doc, ".//xdr:pic", ns = xml_ns(doc))
  expect_is(pic_node, "xml_node")

  blip_node <- xml_find_first(pic_node, ".//a:blip", ns = xml_ns(doc))
  expect_is(blip_node, "xml_node")
})

context("Test DrawingML editable properties")
library(xml2)

test_that("check pptx editable properties", {
  file <- tempfile()
  dml_pptx(file = file, editable = FALSE, bg = "transparent")
  plot.new()
  points(0.2, 0.2)
  dev.off()

  x <- read_xml(file)
  xpath_ <- ".//p:sp/p:nvSpPr/p:cNvSpPr/a:spLocks"
  node <- xml_find_first(x, xpath_, ns = xml_ns(x))
  expect_is(object = node, class = "xml_node")

  file <- tempfile()
  dml_pptx(file = file, editable = TRUE, bg = "transparent")
  plot.new()
  points(0.2, 0.2)
  dev.off()
  x <- read_xml(file)
  xpath_ <- ".//p:sp/p:nvSpPr/p:cNvSpPr/a:spLocks"
  selected_node <- xml_find_first(x, xpath_, ns = xml_ns(x))
  expect_is(selected_node, class = "xml_missing")
})


test_that("check xlsx editable properties", {
  file <- tempfile()
  dml_xlsx(file = file, editable = FALSE, bg = "transparent")
  plot.new()
  points(0.2, 0.2)
  dev.off()

  x <- read_xml(file)
  xpath_ <- ".//xdr:sp/xdr:nvSpPr/xdr:cNvSpPr/a:spLocks"
  node <- xml_find_first(x, xpath_, ns = xml_ns(x))
  expect_is(object = node, class = "xml_node")

  file <- tempfile()
  dml_xlsx(file = file, editable = TRUE, bg = "transparent")
  plot.new()
  points(0.2, 0.2)
  dev.off()
  x <- read_xml(file)
  xpath_ <- ".//xdr:sp/xdr:nvSpPr/xdr:cNvSpPr/a:spLocks"
  selected_node <- xml_find_first(x, xpath_, ns = xml_ns(x))
  expect_is(selected_node, class = "xml_missing")
})

#' @export
#' @title Add a dml drawing to an Excel sheet
#' @description Add a [dml()] object to a sheet in an xlsx workbook
#' created with [officer::read_xlsx()]. The plot is rendered as
#' editable vector graphics (DrawingML).
#' @param x an rxlsx object (created by [officer::read_xlsx()])
#' @param value a [dml()] object
#' @param sheet sheet name (must already exist)
#' @param left,top left and top origin of the plot on the sheet in inches
#' @param width,height width and height in inches
#' @param ... additional arguments passed to [dml_xlsx()]
#' @return the rxlsx object (invisibly)
#' @importFrom officer sheet_add_drawing xlsx_drawing
#' @importFrom xml2 xml_add_child xml_find_all read_xml
#' @method sheet_add_drawing dml
#' @examples
#' \donttest{
#' library(officer)
#' library(rvg)
#'
#' x <- read_xlsx()
#' x <- sheet_add_drawing(x,
#'   value = dml(code = barplot(1:5, col = 2:6)),
#'   sheet = "Feuil1",
#'   left = 1, top = 2, width = 6, height = 6
#' )
#' print(x, target = tempfile(fileext = ".xlsx"))
#' }
sheet_add_drawing.dml <- function(
  x,
  value,
  sheet,
  left = 1,
  top = 1,
  width = 6,
  height = 6,
  ...
) {
  stopifnot(inherits(x, "rxlsx"))

  package_dir <- x$package_dir

  # ensure media dir exists (rasters referenced by the dml go there)
  media_dir <- file.path(package_dir, "xl/media")
  dir.create(media_dir, showWarnings = FALSE, recursive = TRUE)

  # find sheet object
  sheet_idx <- which(x$worksheets$sheet_names() == sheet)
  sheet_obj <- x$sheets$get_sheet(sheet_idx)

  # Get (or create) the shared drawing part for this sheet. Anchors
  # produced below will be appended to it so that a sheet never ends
  # up with more than one <drawing> reference.
  drawing <- officer::xlsx_drawing$new(
    package_dir,
    sheet_obj,
    x$content_type
  )

  # Render the dml to a throw-away file; we only want its <xdr:*Anchor>
  # children and the list of raster files it produced.
  tmp_dml_file <- tempfile(pattern = "rvg_dml_", fileext = ".xml")

  pars <- list(...)
  pars$file <- tmp_dml_file
  pars$id <- 0L
  # Raster rIds written inside the dml XML must slot into the shared
  # drawing's rels, so start numbering from the drawing's next id.
  pars$last_rel_id <- drawing$relationship()$get_next_id() - 1
  pars$standalone <- TRUE
  pars$width <- width
  pars$height <- height
  pars$offx <- left
  pars$offy <- top
  if (!is.null(value$bg)) {
    pars$bg <- value$bg
  }
  if (!is.null(value$fonts) && length(value$fonts) > 0) {
    pars$fonts <- value$fonts
  }
  if (!is.null(value$pointsize)) {
    pars$pointsize <- value$pointsize
  }
  if (!is.null(value$editable)) {
    pars$editable <- value$editable
  }

  do.call("dml_xlsx", pars)

  # execute the plotting code
  if (!is.null(value$ggobj)) {
    print(value$ggobj)
  } else {
    tryCatch(
      eval(
        rlang::quo_get_expr(value$code),
        envir = rlang::quo_get_env(value$code)
      ),
      finally = NULL
    )
  }
  dev.off()

  # Extract the raster file list (if any) from the marker comment.
  xml_raw <- paste0(readLines(tmp_dml_file, warn = FALSE), collapse = "")
  m <- regmatches(
    xml_raw,
    regexpr("<!-- rvg_raster_prefix:(.+?) -->", xml_raw, perl = TRUE)
  )
  raster_files <- character(0)
  if (length(m) == 1L) {
    raster_prefix <- sub(
      "<!-- rvg_raster_prefix:(.+?) -->",
      "\\1",
      m,
      perl = TRUE
    )
    raster_files <- list_raster_files(img_dir = raster_prefix)
  }

  # Copy rasters into xl/media/ and register them as rels on the
  # shared drawing. rIds are assigned in the same order dml_xlsx
  # allocated them (incrementing from `last_rel_id`), so the
  # `r:embed="rIdN"` attributes inside the anchors resolve correctly.
  if (length(raster_files)) {
    file.copy(raster_files, media_dir)
    for (rf in basename(raster_files)) {
      drawing$add_image_rel(rf)
    }
  }

  # Append every anchor produced by dml_xlsx to the shared drawing.
  dml_doc <- read_xml(tmp_dml_file)
  anchors <- xml_find_all(
    dml_doc,
    "xdr:absoluteAnchor | xdr:oneCellAnchor | xdr:twoCellAnchor",
    ns = c(
      xdr = "http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing"
    )
  )
  for (a in anchors) {
    xml_add_child(drawing$get(), a)
  }
  drawing$save()

  unlink(tmp_dml_file)

  invisible(x)
}

list_raster_files <- function(img_dir) {
  path_ <- dirname(img_dir)
  uid <- basename(img_dir)
  list.files(
    path = path_,
    pattern = paste0("^", uid, "(.*)\\.png$"),
    full.names = TRUE
  )
}


#' @title Wrap plot instructions for DrawingML plotting in Powerpoint
#' @description A simple wrapper to mark
#' the plot instructions as Vector Graphics instructions. It produces
#' an object of class 'dml' with a corresponding method \code{\link[officer]{ph_with}}.
#'
#' The function enables usage of any R plot with argument `code` and with
#' ggplot objects with argument `ggobj`.
#'
#' **Note:** the output is a vector graphic, not a native Microsoft Office
#' chart. The underlying data is not embedded in the document.
#' @section background color:
#' When dealing with a ggplot object, the `bg` parameter sets the
#' device background but the ggplot theme's `plot.background` will
#' typically draw over it. To control the background appearance,
#' set it in the theme instead:
#'
#' ```
#' theme(
#'   panel.background = element_rect(fill = "#EFEFEF"),
#'   plot.background = element_rect(fill = "wheat"))
#' ```
#' @section Limitations:
#' The DrawingML format does not support the following R graphics features:
#'
#' - **Patterns and gradients**: grid patterns (`GridPattern`),
#'   linear and radial gradients used as fill are not translated into
#'   DrawingML. ggplot2 will emit a warning when these are used.
#' - **Rich text (ggtext/gridtext)**: packages such as `ggtext` render
#'   HTML/Markdown into composite grid grobs. The rvg device can only
#'   capture low-level primitives (text strings, lines, polygons), so
#'   the rich formatting (inline images, coloured spans, HTML markup)
#'   is lost. Use standard ggplot2 text functions or apply formatting
#'   after export in PowerPoint.
#' - **Plotmath expressions**: R's graphics engine decomposes
#'   `expression()` into individual text primitives before the device
#'   sees them. Compound expressions (e.g. `expression("a" * "b")`,
#'   superscripts, subscripts) are rendered as separate text boxes,
#'   causing vertical misalignment.
#' - **Clipping paths, masks, compositing and transformations**:
#'   these R >= 4.1 features have no DrawingML equivalent and are
#'   silently ignored.
#'
#' @param code plotting instructions
#' @param ggobj ggplot object to print. argument code will be ignored if this
#'   argument is supplied.
#' @param bg,fonts,pointsize,editable Parameters stored and
#'   later passed to \code{\link{dml_pptx}} by \code{\link{ph_with.dml}}.
#' @param ... unused arguments
#' @examples
#' anyplot <- dml(code = barplot(1:5, col = 2:6), bg = "wheat")
#'
#' library(officer)
#' doc <- read_pptx()
#' doc <- add_slide(doc, "Title and Content", "Office Theme")
#' doc <- ph_with(doc, anyplot, location = ph_location_fullsize())
#' fileout <- tempfile(fileext = ".pptx")
#' # fileout <- "vg.pptx"
#' print(doc, target = fileout)
#' @export
#' @seealso \code{\link{ph_with.dml}}
dml <- function(
  code,
  ggobj = NULL,
  bg = "white",
  fonts = list(),
  pointsize = 12,
  editable = TRUE,
  ...
) {
  out <- list()
  out$code <- enquo(code)
  out$ggobj <- ggobj
  out$bg <- bg
  out$fonts <- fonts
  out$pointsize <- pointsize
  out$editable <- editable
  class(out) <- "dml"
  return(out)
}

#' @importFrom xml2 xml_find_first as_xml_document
#' @importFrom officer ph_location ph_location_type
#' @importFrom officer ph_with fortify_location
#' @importFrom rlang eval_tidy enquo
#' @export
#' @method ph_with dml
#' @title add a plot output as vector graphics into a PowerPoint object
#' @description Produces a vector graphics output from R plot instructions
#' stored in a \code{\link{dml}} object and adds the result in an \code{rpptx}
#' object produced by \code{\link[officer]{read_pptx}}.
#' @param x a pptx device
#' @param value \code{\link{dml}} object
#' @param location a location for a placeholder.
#' @param ... Arguments to be passed to methods
#' @examples
#' anyplot <- dml(code = barplot(1:5, col = 2:6), bg = "wheat")
#'
#' library(officer)
#' doc <- read_pptx()
#' doc <- add_slide(doc, "Title and Content", "Office Theme")
#' doc <- ph_with(doc, anyplot, location = ph_location_fullsize())
#'
#' fileout <- tempfile(fileext = ".pptx")
#' print(doc, target = fileout)
ph_with.dml <- function(x, value, location, ...) {
  img_directory <- tempfile()
  dir.create(img_directory, recursive = TRUE, showWarnings = FALSE)

  dml_file <- tempfile()

  pars <- list(...)

  loc <- fortify_location(location, doc = x)

  pars$offx <- loc$left
  pars$offy <- loc$top
  pars$width <- loc$width
  pars$height <- loc$height

  pars$bg <- value$bg
  pars$fonts <- value$fonts
  pars$pointsize <- value$pointsize
  pars$editable <- value$editable

  pars$file <- dml_file
  pars$id <- 0L
  pars$last_rel_id <- 1
  pars$raster_prefix <- paste0(img_directory, "/raster-")
  pars$standalone <- FALSE

  do.call("dml_pptx", pars)

  tryCatch(
    {
      if (!is.null(value$ggobj)) {
        stopifnot(inherits(value$ggobj, "ggplot"))
        print(value$ggobj)
      } else {
        eval_tidy(value$code)
      }
    },
    finally = dev.off()
  )

  dml_str <- scan(
    dml_file,
    what = "character",
    quiet = T,
    sep = "\n",
    encoding = "UTF-8"
  )

  if (dml_str == "</p:grpSp>") {
    stop(
      "There was no plot output produced, cannot add an empty plot to pptx document."
    )
  }

  dml_str <- paste(dml_str, collapse = "")
  ph_with(
    x = x,
    value = xml2::as_xml_document(dml_str),
    location = location,
    ...
  )
}

context("add vg in pptx")
library(xml2)
library(officer)

test_that("ph_with", {
  doc <- read_pptx()
  doc <- add_slide(doc, "Title and Content", "Office Theme")
  doc <- ph_with(
    doc,
    dml(code = barplot(1:5, col = 2:6)),
    location = ph_location_type(type = "body")
  )

  xmldoc <- doc$slide$get_slide(1)$get()
  node <- xml_find_all(xmldoc, "//p:spTree/p:grpSp/*")
  expect_false(inherits(node, "xml_missing"))
})

test_that("ph_remove", {
  my_pres <- read_pptx()
  #slide 1
  my_pres <- add_slide(
    my_pres,
    layout = "Title and Content",
    master = "Office Theme"
  )
  my_pres <- ph_with(
    my_pres,
    value = "Hello World",
    location = ph_location_type(type = "title", newlabel = "ouaoua")
  )
  my_pres <- ph_with(
    my_pres,
    value = "A footer",
    location = ph_location_type(type = "ftr", newlabel = "ouaoui")
  )
  my_pres <- ph_with(
    my_pres,
    value = format(Sys.Date()),
    location = ph_location_type(type = "dt", newlabel = "ouaoue")
  )
  #slide 2
  my_pres <- add_slide(my_pres, layout = "Title and Content")
  my_pres <- ph_with(
    my_pres,
    value = dml(code = {
      boxplot(iris)
    }),
    location = ph_location_type("body", newlabel = "chart")
  )

  example_pptx <- print(my_pres, target = tempfile(fileext = ".pptx"))

  # Does not remove graphic elements currently
  mypres2 <- read_pptx(example_pptx)
  mypres2 <- on_slide(mypres2, index = 1)
  mypres2 <- ph_remove(mypres2, ph_label = "ouaoua")
  mypres2 <- ph_remove(mypres2, ph_label = "ouaoui")
  mypres2 <- ph_remove(mypres2, ph_label = "ouaoue")
  mypres2 <- on_slide(mypres2, index = 2)
  mypres2 <- ph_remove(mypres2, ph_label = "chart")

  ss_1 <- slide_summary(mypres2, index = 1)
  ss_2 <- slide_summary(mypres2, index = 2)

  expect_equal(nrow(ss_1), 0L)
  expect_equal(nrow(ss_2), 0L)
})

#' @export
#' @keywords internal
#' @title Defunct Functions in Package rvg
#' @param ... unused argument
#' @details `body_add_vg()` is removed and can be replaced by `officer::body_add_img`.
body_add_vg <- function(...) {
  .Defunct("officer::body_add_img")
}

#' @useDynLib rvg,.registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @import systemfonts

r_font_families <- c("sans", "serif", "mono", "symbol")

#' @importFrom gdtools font_family_exists font_set_auto
validate_fonts <- function(system_fonts = list()) {
  auto <- font_set_auto()
  defaults <- c(
    sans = auto$sans,
    serif = auto$serif,
    mono = auto$mono,
    symbol = auto$symbol
  )
  found <- unlist(lapply(system_fonts, font_family_exists))
  if (length(system_fonts) > 0 && any(!found)) {
    not_found <- names(system_fonts)[!found]
    warning(
      "Font families not found on this system and replaced by defaults: ",
      paste0('"', not_found, '"', collapse = ", "),
      ". Use gdtools::font_family_exists() to check availability.",
      call. = FALSE
    )
  }
  system_fonts <- system_fonts[found]
  missing_fonts <- setdiff(r_font_families, names(system_fonts))
  system_fonts[missing_fonts] <- defaults[missing_fonts]
  system_fonts
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

PPTX_ <- function(file, bg_, width, height, offx, offy, pointsize, aliases, editable, id, raster_prefix, last_rel_id, standalone) {
    .Call(`_rvg_PPTX_`, file, bg_, width, height, offx, offy, pointsize, aliases, editable, id, raster_prefix, last_rel_id, standalone)
}

XLSX_ <- function(file, bg_, width, height, offx, offy, pointsize, aliases, editable, id, raster_prefix, last_rel_id, standalone) {
    .Call(`_rvg_XLSX_`, file, bg_, width, height, offx, offy, pointsize, aliases, editable, id, raster_prefix, last_rel_id, standalone)
}

context("Background color")
library(xml2)

test_that("pptx background exists if background is not transparent", {
  file <- tempfile()
  dml_pptx(file = file, bg = "#123456")
  plot.new()
  dev.off()

  doc <- read_xml(file)
  solid_fill <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill",
    ns = xml_ns(doc)
  )
  expect_is(object = solid_fill, class = "xml_node")

  color_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill/a:srgbClr",
    ns = xml_ns(doc)
  )
  color <- xml_attr(color_node, "val")
  expect_equal(object = color, expected = "123456")

  alpha_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill/a:srgbClr/a:alpha",
    ns = xml_ns(doc)
  )
  alpha <- xml_attr(alpha_node, "val")
  expect_equal(object = alpha, expected = "100000")

  file <- tempfile()
  dml_pptx(file = file, bg = "#12345699")
  plot.new()
  dev.off()

  doc <- read_xml(file)
  alpha_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill/a:srgbClr/a:alpha",
    ns = xml_ns(doc)
  )
  alpha <- xml_attr(alpha_node, "val")
  expect_equal(object = alpha, expected = "60000")
})


test_that("pptx background does not exist if background is transparent", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  dev.off()

  doc <- read_xml(file)
  bg_node <- xml_find_first(doc, ".//p:sp", ns = xml_ns(doc))
  expect_is(bg_node, class = "xml_missing")
})

context("PPTX text")
library(xml2)
library(gdtools)

test_that("text can be found", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(0.2, 0.2, "hello")
  dev.off()

  doc <- read_xml(file)
  text_node <- xml_find_first(
    doc,
    ".//p:sp/p:txBody/a:p/a:r/a:t",
    ns = xml_ns(doc)
  )
  expect_is(object = text_node, class = "xml_node")
  expect_equal(xml_text(text_node), "hello")
})

test_that("cex affects strwidth", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  w1 <- strwidth("X")
  par(cex = 4)
  w4 <- strwidth("X")
  dev.off()
  expect_equal(w4 / w1, 4, tol = 1e-3)
})

test_that("special characters are escaped", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(0.5, 0.5, "<&>")
  dev.off()

  x <- read_xml(file)
  ns <- xml_ns(x)
  expect_equal(
    xml_text(xml_find_first(x, ".//p:sp/p:txBody/a:p/a:r/a:t", ns = ns)),
    "<&>"
  )
})

test_that("utf-8 characters are preserved", {
  skip_on_os("windows") # skip because of xml2 buglet

  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(0.5, 0.5, "\u00b5")
  dev.off()

  x <- read_xml(file)
  ns <- xml_ns(x)
  expect_equal(
    xml_text(xml_find_first(x, ".//p:sp/p:txBody/a:p/a:r/a:t", ns = ns)),
    "\u00b5"
  )
})

test_that("text color is written in fill attr", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(0.5, 0.5, "a", col = "#113399")
  dev.off()

  x <- read_xml(file)
  ns <- xml_ns(x)
  expect_equal(
    xml_attr(
      xml_find_first(
        x,
        ".//p:sp/p:txBody/a:p/a:r/a:rPr/a:solidFill/a:srgbClr",
        ns = ns
      ),
      "val"
    ),
    "113399"
  )
})

test_that("default point size is 12", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(0.5, 0.5, "a")
  dev.off()

  x <- read_xml(file)
  ns <- xml_ns(x)
  rPr <- xml_find_first(x, ".//p:sp/p:txBody/a:p/a:r/a:rPr", ns = ns)
  expect_equal(xml_attr(rPr, "sz"), "1200")
})

test_that("cex does not generate fractional font sizes", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(0.5, 0.5, "a", cex = .1)
  dev.off()

  x <- read_xml(file)
  ns <- xml_ns(x)
  rPr <- xml_find_first(x, ".//p:sp/p:txBody/a:p/a:r/a:rPr", ns = ns)
  expect_equal(xml_attr(rPr, "sz"), "120")
})

test_that("font sets weight/style", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(0.5, seq(0.9, 0.1, length = 4), "a", font = 1:4)
  dev.off()

  x <- read_xml(file)
  ns <- xml_ns(x)
  rPr <- xml_find_all(x, ".//p:sp/p:txBody/a:p/a:r/a:rPr", ns = ns)
  expect_equal(xml_attr(rPr, "b"), c("0", "1", "0", "1"))
  expect_equal(xml_attr(rPr, "i"), c("0", "0", "1", "1"))
})


test_that("font sets weight/style", {
  skip_if_not(font_family_exists("Arial"))
  skip_if_not(font_family_exists("Times New Roman"))
  skip_if_not(font_family_exists("Courier New"))

  file <- tempfile()
  dml_pptx(
    file = file,
    bg = "transparent",
    fonts = list(
      sans = "Arial",
      serif = "Times New Roman",
      mono = "Courier New"
    )
  )
  plot.new()
  text(0.5, 0.1, "a", family = "serif")
  text(0.5, 0.5, "a", family = "sans")
  text(0.5, 0.9, "a", family = "mono")
  dev.off()

  x <- read_xml(file)
  ns <- xml_ns(x)
  rPr_latin <- xml_find_all(
    x,
    ".//p:sp/p:txBody/a:p/a:r/a:rPr/a:latin",
    ns = ns
  )
  rPr_cs <- xml_find_all(x, ".//p:sp/p:txBody/a:p/a:r/a:rPr/a:cs", ns = ns)
  expect_equal(
    xml_attr(rPr_latin, "typeface"),
    c("Times New Roman", "Arial", "Courier New")
  )
  expect_equal(
    xml_attr(rPr_cs, "typeface"),
    c("Times New Roman", "Arial", "Courier New")
  )
})

test_that("a symbol has width greater than 0", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot(c(0, 2), c(0, 2), type = "n")
  strw <- strwidth(expression(symbol("\042")))
  dev.off()

  expect_gt(strw, 0)
})

context("PPTX shape dimensions")
library(xml2)

test_that("rect has dimensions", {
  file <- tempfile()

  dml_pptx(file = file, bg = "transparent")
  plot.new()
  rect(0.2, 0.2, 0.8, 0.8)
  dev.off()

  doc <- read_xml(file)
  xfrm_node <- xml_find_first(doc, ".//p:sp/p:spPr/a:xfrm", ns = xml_ns(doc))
  expect_is(object = xfrm_node, class = "xml_node")

  off_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:off",
    ns = xml_ns(doc)
  )
  offx <- xml_attr(off_node, "x")
  offy <- xml_attr(off_node, "y")
  expect_true(grepl("^[0-9]+$", offx))
  expect_true(grepl("^[0-9]+$", offy))

  ext_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:ext",
    ns = xml_ns(doc)
  )
  cx <- xml_attr(ext_node, "cx")
  cy <- xml_attr(ext_node, "cy")
  expect_true(grepl("^[0-9]+$", cx))
  expect_true(grepl("^[0-9]+$", cy))
})

test_that("lines has dimensions", {
  file <- tempfile()

  dml_pptx(file = file, bg = "transparent")
  plot.new()
  lines(par("usr")[1:2], par("usr")[3:4], lwd = 1)
  dev.off()

  doc <- read_xml(file)
  xfrm_node <- xml_find_first(doc, ".//p:sp/p:spPr/a:xfrm", ns = xml_ns(doc))
  expect_is(object = xfrm_node, class = "xml_node")

  off_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:off",
    ns = xml_ns(doc)
  )
  offx <- xml_attr(off_node, "x")
  offy <- xml_attr(off_node, "y")
  expect_true(grepl("^[0-9]+$", offx))
  expect_true(grepl("^[0-9]+$", offy))

  ext_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:ext",
    ns = xml_ns(doc)
  )
  cx <- xml_attr(ext_node, "cx")
  cy <- xml_attr(ext_node, "cy")
  expect_true(grepl("^[0-9]+$", cx))
  expect_true(grepl("^[0-9]+$", cy))
})

test_that("polygon has dimensions", {
  file <- tempfile()

  dml_pptx(file = file, bg = "transparent")
  plot.new()
  x <- par("usr")[1:2]
  y <- par("usr")[3:4]
  x <- c(x[1], x[1], x[2], x[2], NA)
  y <- c(y[2], y[1], y[1], y[2], NA)
  polygon(x, y, lwd = 1)
  dev.off()

  doc <- read_xml(file)
  xfrm_node <- xml_find_first(doc, ".//p:sp/p:spPr/a:xfrm", ns = xml_ns(doc))
  expect_is(object = xfrm_node, class = "xml_node")

  off_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:off",
    ns = xml_ns(doc)
  )
  offx <- xml_attr(off_node, "x")
  offy <- xml_attr(off_node, "y")
  expect_true(grepl("^[0-9]+$", offx))
  expect_true(grepl("^[0-9]+$", offy))

  ext_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:ext",
    ns = xml_ns(doc)
  )
  cx <- xml_attr(ext_node, "cx")
  cy <- xml_attr(ext_node, "cy")
  expect_true(grepl("^[0-9]+$", cx))
  expect_true(grepl("^[0-9]+$", cy))
})


test_that("text has dimensions", {
  file <- tempfile()

  dml_pptx(file = file, bg = "transparent")
  plot.new()
  text(x = .5, y = .5, labels = "test")
  dev.off()

  doc <- read_xml(file)
  xfrm_node <- xml_find_first(doc, ".//p:sp/p:spPr/a:xfrm", ns = xml_ns(doc))
  expect_is(object = xfrm_node, class = "xml_node")

  off_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:off",
    ns = xml_ns(doc)
  )
  offx <- xml_attr(off_node, "x")
  offy <- xml_attr(off_node, "y")
  expect_true(grepl("^[0-9]+$", offx))
  expect_true(grepl("^[0-9]+$", offy))

  ext_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:xfrm/a:ext",
    ns = xml_ns(doc)
  )
  cx <- xml_attr(ext_node, "cx")
  cy <- xml_attr(ext_node, "cy")
  expect_true(grepl("^[0-9]+$", cx))
  expect_true(grepl("^[0-9]+$", cy))
})

context("PPTX Lines")
library(xml2)

test_that("segments don't have fill", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  segments(0.5, 0.5, 1, 1)
  dev.off()

  doc <- read_xml(file)
  fill_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill",
    ns = xml_ns(doc)
  )
  expect_is(fill_node, "xml_missing")
})

test_that("lines don't have fill", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  lines(c(0.5, 1, 0.5), c(0.5, 1, 1))
  dev.off()

  doc <- read_xml(file)
  fill_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill",
    ns = xml_ns(doc)
  )
  expect_is(fill_node, "xml_missing")
})

test_that("polygons do have fill", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  polygon(c(0.5, 1, 0.5), c(0.5, 1, 1), col = "red", border = "blue")
  dev.off()

  doc <- read_xml(file)
  fill_node <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill",
    ns = xml_ns(doc)
  )
  expect_is(fill_node, "xml_node")
})

test_that("polygons without border", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  polygon(c(0.5, 1, 0.5), c(0.5, 1, 1), col = "red", border = NA)
  dev.off()

  doc <- read_xml(file)
  fill_color <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:solidFill/a:srgbClr",
    ns = xml_ns(doc)
  )
  expect_equal(xml_attr(fill_color, "val"), "FF0000")

  line_color <- xml_find_first(doc, ".//p:sp/p:spPr/a:ln", ns = xml_ns(doc))
  expect_is(line_color, "xml_missing")
})

dash_array <- function(...) {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot(1:3, ..., axes = FALSE, xlab = "", ylab = "", type = "l")
  dev.off()

  doc <- read_xml(file)
  dash <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:ln/a:prstDash",
    ns = xml_ns(doc)
  )
  dash
}
custom_dash_array <- function(...) {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot(1:3, ..., axes = FALSE, xlab = "", ylab = "", type = "l")
  dev.off()

  doc <- read_xml(file)
  dash <- xml_find_all(
    doc,
    ".//p:sp/p:spPr/a:ln/a:custDash/a:ds",
    ns = xml_ns(doc)
  )
  as.character(unlist(lapply(dash, xml_attrs)))
}

test_that("lty are ok", {
  expect_equal(xml_attr(dash_array(lty = 1), "val"), "solid")
  expect_equal(xml_attr(dash_array(lty = 2), "val"), "dash")
  expect_equal(xml_attr(dash_array(lty = 3), "val"), "dot")
  expect_equal(
    custom_dash_array(lty = 4),
    c("100000", "300000", "400000", "300000")
  )
  expect_equal(xml_attr(dash_array(lty = 5), "val"), "lgDash")
  expect_equal(
    custom_dash_array(lty = 6),
    c("200000", "200000", "600000", "200000")
  )
  expect_equal(custom_dash_array(lty = "1F"), c("100000", "1500000"))
  expect_equal(
    custom_dash_array(lty = "1234"),
    c("100000", "200000", "300000", "400000")
  )
})

test_that("lty scales with lwd", {
  expect_equal(
    custom_dash_array(lty = 4),
    c("100000", "300000", "400000", "300000")
  )
  expect_equal(
    custom_dash_array(lty = 4, lwd = 2),
    c("200000", "600000", "800000", "600000")
  )
})


test_that("line join shapes", {
  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  lines(c(0.3, 0.5, 0.7), c(0.1, 0.9, 0.1), lwd = 15, ljoin = "round")
  dev.off()

  doc <- read_xml(file)
  join_shape <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:ln/a:round",
    ns = xml_ns(doc)
  )
  expect_is(join_shape, "xml_node")

  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  lines(c(0.3, 0.5, 0.7), c(0.1, 0.9, 0.1), lwd = 15, ljoin = "mitre")
  dev.off()

  doc <- read_xml(file)
  join_shape <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:ln/a:miter",
    ns = xml_ns(doc)
  )
  expect_is(join_shape, "xml_node")

  file <- tempfile()
  dml_pptx(file = file, bg = "transparent")
  plot.new()
  lines(c(0.3, 0.5, 0.7), c(0.1, 0.9, 0.1), lwd = 15, ljoin = "bevel")
  dev.off()

  doc <- read_xml(file)
  join_shape <- xml_find_first(
    doc,
    ".//p:sp/p:spPr/a:ln/a:bevel",
    ns = xml_ns(doc)
  )
  expect_is(join_shape, "xml_node")
})

context("PPTX raster generation")
library(grid)
library(xml2)

test_that("raster is generated", {
  file <- tempfile()
  uid <- basename(tempfile())
  dml_pptx(file = file, raster_prefix = uid, bg = "transparent")
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp1"))
  grid.rect()
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp2", angle = 10))
  redGradient <- matrix(hcl(0, 80, seq(50, 80, 10)), nrow = 4, ncol = 5)
  grid.raster(redGradient)
  dev.off()

  raster_files <- list.files(
    pattern = paste0("^", uid, "(.*)\\.png$"),
    full.names = TRUE
  )
  expect_equal(length(raster_files), expected = 1)
})


test_that("pic tag can be found", {
  file <- tempfile()

  dml_pptx(file = file, bg = "transparent")
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp1"))
  grid.rect()
  pushViewport(viewport(width = 0.8, height = 0.5, name = "vp2", angle = 10))
  redGradient <- matrix(hcl(0, 80, seq(50, 80, 10)), nrow = 4, ncol = 5)
  grid.raster(redGradient)
  dev.off()

  doc <- read_xml(file)
  pic_node <- xml_find_first(doc, ".//p:pic", ns = xml_ns(doc))
  expect_is(pic_node, "xml_node")

  blip_node <- xml_find_first(pic_node, ".//a:blip", ns = xml_ns(doc))
  expect_is(blip_node, "xml_node")
})

unlink(list.files(pattern = "\\.png"), force = TRUE)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xl_add_vg.R
\name{xl_add_vg}
\alias{xl_add_vg}
\title{add a plot output as vector graphics into an Excel object}
\usage{
xl_add_vg(x, sheet, code, left, top, width, height, ...)
}
\arguments{
\item{x}{an \code{rxlsx} object produced by \code{officer::read_xlsx}}

\item{sheet}{sheet label/name}

\item{code}{plot instructions}

\item{left, top}{left and top origin of the plot on the sheet in inches.}

\item{height, width}{Height and width in inches.}

\item{...}{arguments passed on to \code{\link{dml_xlsx}}.}
}
\description{
Deprecated, use \code{\link[officer:sheet_add_drawing]{officer::sheet_add_drawing()}} with a \code{\link[=dml]{dml()}} object instead.
}
\examples{
\donttest{
library(officer)
my_ws <- read_xlsx()
my_ws <- sheet_add_drawing(my_ws,
  value = dml(code = barplot(1:5, col = 2:6)),
  sheet = "Feuil1",
  width = 6, height = 6, left = 1, top = 2
)
fileout <- tempfile(fileext = ".xlsx")
print(my_ws, target = fileout)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ph_with_vg.R
\name{ph_with.dml}
\alias{ph_with.dml}
\title{add a plot output as vector graphics into a PowerPoint object}
\usage{
\method{ph_with}{dml}(x, value, location, ...)
}
\arguments{
\item{x}{a pptx device}

\item{value}{\code{\link{dml}} object}

\item{location}{a location for a placeholder.}

\item{...}{Arguments to be passed to methods}
}
\description{
Produces a vector graphics output from R plot instructions
stored in a \code{\link{dml}} object and adds the result in an \code{rpptx}
object produced by \code{\link[officer]{read_pptx}}.
}
\examples{
anyplot <- dml(code = barplot(1:5, col = 2:6), bg = "wheat")

library(officer)
doc <- read_pptx()
doc <- add_slide(doc, "Title and Content", "Office Theme")
doc <- ph_with(doc, anyplot, location = ph_location_fullsize())

fileout <- tempfile(fileext = ".pptx")
print(doc, target = fileout)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sheet_add_drawing_dml.R
\name{sheet_add_drawing.dml}
\alias{sheet_add_drawing.dml}
\title{Add a dml drawing to an Excel sheet}
\usage{
\method{sheet_add_drawing}{dml}(
  x,
  value,
  sheet,
  left = 1,
  top = 1,
  width = 6,
  height = 6,
  ...
)
}
\arguments{
\item{x}{an rxlsx object (created by \code{\link[officer:read_xlsx]{officer::read_xlsx()}})}

\item{value}{a \code{\link[=dml]{dml()}} object}

\item{sheet}{sheet name (must already exist)}

\item{left, top}{left and top origin of the plot on the sheet in inches}

\item{width, height}{width and height in inches}

\item{...}{additional arguments passed to \code{\link[=dml_xlsx]{dml_xlsx()}}}
}
\value{
the rxlsx object (invisibly)
}
\description{
Add a \code{\link[=dml]{dml()}} object to a sheet in an xlsx workbook
created with \code{\link[officer:read_xlsx]{officer::read_xlsx()}}. The plot is rendered as
editable vector graphics (DrawingML).
}
\examples{
\donttest{
library(officer)
library(rvg)

x <- read_xlsx()
x <- sheet_add_drawing(x,
  value = dml(code = barplot(1:5, col = 2:6)),
  sheet = "Feuil1",
  left = 1, top = 2, width = 6, height = 6
)
print(x, target = tempfile(fileext = ".xlsx"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dml_xlsx.R
\name{dml_xlsx}
\alias{dml_xlsx}
\title{DrawingML graphic device for Microsoft Excel}
\usage{
dml_xlsx(
  file = "Rplots.dml",
  width = 6,
  height = 6,
  offx = 1,
  offy = 1,
  bg = "white",
  fonts = list(),
  pointsize = 12,
  editable = TRUE,
  id = 1L,
  last_rel_id = 1L,
  standalone = TRUE
)
}
\arguments{
\item{file}{the file where output will appear.}

\item{height, width}{Height and width in inches.}

\item{offx, offy}{top and left origin of the plot}

\item{bg}{Default background color for the plot (defaults to "white").}

\item{fonts}{Named list of font names to be aliased with
fonts installed on your system. If unspecified, the R default
families \code{sans}, \code{serif}, \code{mono} and \code{symbol} are aliased
to the families detected by \code{\link[gdtools:font_set_auto]{gdtools::font_set_auto()}}.

\strong{Important:} fonts are referenced by name in the DrawingML output
and are \strong{not embedded}. The reader of the Excel file must also
have the same fonts installed, otherwise Office will substitute them.
For maximum portability, prefer fonts bundled with Microsoft Office
such as \code{"Arial"}, \code{"Times New Roman"} and \code{"Courier New"}.

Fonts must be installed on your system; use
\code{gdtools::font_family_exists()} to verify availability.

Example: \code{list(sans = "Arial", serif = "Times New Roman", mono = "Courier New")}.}

\item{pointsize}{default point size.}

\item{editable}{should vector graphics elements (points, text, etc.) be editable.}

\item{id}{specifies a unique identifier (integer) within the
sheet that will contain the DrawingML instructions.}

\item{last_rel_id}{specifies the last unique identifier (integer)
within relationship file that will be used to reference embedded
raster images if any.}

\item{standalone}{produce a standalone drawingml file? If \code{FALSE}, omits
xml header and namespaces.}
}
\description{
Graphics devices for Microsoft Excel DrawingML format.
}
\examples{
\donttest{
dml_xlsx(file = tempfile())
plot(1:11, (-5:5)^2, type = "b", main = "Simple Example")
dev.off()
}
}
\seealso{
\code{\link{Devices}}
}
\keyword{device}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ph_with_vg.R
\name{dml}
\alias{dml}
\title{Wrap plot instructions for DrawingML plotting in Powerpoint}
\usage{
dml(
  code,
  ggobj = NULL,
  bg = "white",
  fonts = list(),
  pointsize = 12,
  editable = TRUE,
  ...
)
}
\arguments{
\item{code}{plotting instructions}

\item{ggobj}{ggplot object to print. argument code will be ignored if this
argument is supplied.}

\item{bg, fonts, pointsize, editable}{Parameters stored and
later passed to \code{\link{dml_pptx}} by \code{\link{ph_with.dml}}.}

\item{...}{unused arguments}
}
\description{
A simple wrapper to mark
the plot instructions as Vector Graphics instructions. It produces
an object of class 'dml' with a corresponding method \code{\link[officer]{ph_with}}.

The function enables usage of any R plot with argument \code{code} and with
ggplot objects with argument \code{ggobj}.

\strong{Note:} the output is a vector graphic, not a native Microsoft Office
chart. The underlying data is not embedded in the document.
}
\section{background color}{

When dealing with a ggplot object, the \code{bg} parameter sets the
device background but the ggplot theme's \code{plot.background} will
typically draw over it. To control the background appearance,
set it in the theme instead:

\if{html}{\out{<div class="sourceCode">}}\preformatted{theme(
  panel.background = element_rect(fill = "#EFEFEF"),
  plot.background = element_rect(fill = "wheat"))
}\if{html}{\out{</div>}}
}

\section{Limitations}{

The DrawingML format does not support the following R graphics features:
\itemize{
\item \strong{Patterns and gradients}: grid patterns (\code{GridPattern}),
linear and radial gradients used as fill are not translated into
DrawingML. ggplot2 will emit a warning when these are used.
\item \strong{Rich text (ggtext/gridtext)}: packages such as \code{ggtext} render
HTML/Markdown into composite grid grobs. The rvg device can only
capture low-level primitives (text strings, lines, polygons), so
the rich formatting (inline images, coloured spans, HTML markup)
is lost. Use standard ggplot2 text functions or apply formatting
after export in PowerPoint.
\item \strong{Plotmath expressions}: R's graphics engine decomposes
\code{expression()} into individual text primitives before the device
sees them. Compound expressions (e.g. \code{expression("a" * "b")},
superscripts, subscripts) are rendered as separate text boxes,
causing vertical misalignment.
\item \strong{Clipping paths, masks, compositing and transformations}:
these R >= 4.1 features have no DrawingML equivalent and are
silently ignored.
}
}

\examples{
anyplot <- dml(code = barplot(1:5, col = 2:6), bg = "wheat")

library(officer)
doc <- read_pptx()
doc <- add_slide(doc, "Title and Content", "Office Theme")
doc <- ph_with(doc, anyplot, location = ph_location_fullsize())
fileout <- tempfile(fileext = ".pptx")
# fileout <- "vg.pptx"
print(doc, target = fileout)
}
\seealso{
\code{\link{ph_with.dml}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defunct.R
\name{body_add_vg}
\alias{body_add_vg}
\title{Defunct Functions in Package rvg}
\usage{
body_add_vg(...)
}
\arguments{
\item{...}{unused argument}
}
\description{
Defunct Functions in Package rvg
}
\details{
\code{body_add_vg()} is removed and can be replaced by \code{officer::body_add_img}.
}
\keyword{internal}
